==> alg-example-1.56.54-sources.jar: cn/com/duiba/nezha/alg/example/util/demo.java <==

package cn.com.duiba.nezha.alg.example.util;

import cn.com.duiba.nezha.alg.common.enums.DateStyle;
import cn.com.duiba.nezha.alg.common.util.DataUtil;
import cn.com.duiba.nezha.alg.common.util.LocalDateUtil;
import cn.com.duiba.nezha.alg.common.util.MathUtil;
import cn.com.duiba.nezha.alg.example.util.JedisUtil;
import cn.com.duiba.nezha.alg.example.util.conf.JedisPoolConf;
import cn.com.duiba.nezha.alg.common.util.StringZIP;
import cn.com.duiba.nezha.alg.feature.coder.FeatureCoderBase;
import cn.com.duiba.nezha.alg.feature.util.HashUtil;
import com.alibaba.fastjson.JSON;
import cn.com.duiba.nezha.alg.example.util.conf.JedisConfig;
import redis.clients.jedis.Jedis;

import java.util.*;

import static cn.com.duiba.nezha.alg.feature.coder.FeatureCoderBase.getVectorSize;
import static java.util.stream.Collectors.joining;

public class demo {

    private static int F_MAX_SIZE = 64;

    public static int[] seed = {3, 7, 11, 17, 19, 31, 41};
    public static int getSubFId(String feature, String str, int size) {
        int ret = 0;
        str = std(str);

        try {
            if (str != null) {
                Integer tmp = Integer.valueOf(str);

                if (tmp != null && tmp >= 0) {
//                System.out.println(((size - 1) + tmp.intValue()) & (size - 1));
                    ret = ((size - 1) + tmp) % (size - 1);
                    ret += 1;
                }
            }
        } catch (Exception e) {
            System.out.println("errot");
        }

//        addHashConflictTest(feature, str, ret + 0L);
        return ret;
    }



//    public static int[] Arrandlist() {
//        List<String> list = new ArrayList<>();
//        list.add("apple"); // size=1
//        list.add("pear"); // size=2
//        list.add("apple"); // 允许重复添加元素，size=3
//
//
////        List<String> list = List.of("apple", "pear", "banana");
//
//    }


    public static int[] getSubFIdHash(String str, int size) {
        int nums = 1;
        str = std(str);
        int[] ret = new int[nums];
        for (int i = 0; i < nums; i++) {
            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);

//            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);

//            addHashConflictTest(feature, str, ret[i] + 0L);
        }
        Arrays.sort(ret);


        return ret;
    }


    public static int hash(String value,int size,int seed) {
        int result = 0;
        if (value == null) {
            value = "";
        }
        int len = value.length();
        for (int i = 0; i < len; i++) {
            result = seed * result + value.charAt(i);
        }

        return (size - 1) & result;
    }

    public static int[] getHashSubFId(String feature, String str, int size, int nums) throws Exception {

        if (nums > seed.length) {
            return null;
        }
        str = std(str);
        int[] ret = new int[nums];
        for (int i = 0; i < nums; i++) {
            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);

//            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);

//            addHashConflictTest(feature, str, ret[i] + 0L);
        }
        Arrays.sort(ret);


        return ret;
    }

    public static Set<Integer> getIds(int fSize, int pFNums, int[] sFIds, List<Integer> pLenList, List<Integer> indices, List<Double> values) throws Exception {
        Set<Integer> ret = new HashSet<>();
        int pLen = getVectorSize(pLenList);
        if (sFIds != null && sFIds.length > 0) {
            double weight = MathUtil.reciprocalOfN(sFIds.length);
            for (int i = 0; i < sFIds.length; i++) {
                Integer newId = pLen + sFIds[i];
                indices.add(newId);
                ret.add(newId);
                values.add(weight);
            }
        }
        pLenList.add(pLen + pFNums * fSize);
        return ret;
    }

    public static void main(String[] args) throws Exception {


//
//        JedisConfig jedisConf = new JedisConfig();
//        jedisConf.setIp("test.config.dui88.com");
//        jedisConf.setPort(6379);
//        jedisConf.setPassWord("duiba123");
//
//        JedisUtil jedis = new JedisUtil(jedisConf);
//        jedis.set("chengpeng001", "good");
//        System.out.println(jedis.get("chengpeng001"));







//        List<Integer> indices2 = Arrays.asList(1, 2, 3);
//        String b = indices2.toString();
//        System.out.println(b);
//        System.exit(1);

//        Set<Integer> s = new HashSet<>();
//        s.add(1);
//        s.add(3);
//        System.out.println(s.stream().map(String::valueOf).collect(joining("_")));
//        System.exit(1);

//        System.out.println(HashUtil.hash("10001002", 10000, seed[0]));
//        System.exit(0);
//
//
////        String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        List<Integer> indices = new ArrayList<>(F_MAX_SIZE);//field起始+field内编码
//        List<Double> values = new ArrayList<>(F_MAX_SIZE);
//        List<String> fields = new ArrayList<>(F_MAX_SIZE);
//        List<Integer> pLenList = new ArrayList<>(F_MAX_SIZE);//field起始
//        Set<Integer> ret = null;
//        ret = FeatureCoderBase.getHashSubId("f1", "119010102", 100, 1, pLenList, indices, values);
//        System.out.println(ret);
//        System.exit(1);

////        System.out.println(LocalDateUtil.getDayOfWeek("2021-12-21 17:39:18", DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
//        String[] vals = "null,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86".split(",");
//////        String[] vals = "null,0,1,2,3,4".split(",");
//////        String[] vals = "null,1162003,6208378,1603690,799304,442719,291344,203217,145671,108196,81844,63950,51597,157175,77654,75228,56720,44393,15661".split(",");
//////////                String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        String[] vals  = "-1,9427,9152,9277,8755,9326,9427,9277,9326,9277,9277,9431,9431,9277,9427,8743,9277,8743,8755,9427,9277,9311,8755,9372,9277,9427,9277,9326,9277,9277,9311".split(",");
//        for (int i = 0; i < vals.length; i++) {
//            System.out.println(vals[i] + " " +  getSubFId("f840110", vals[i], 100));
//        }
//        System.exit(0);

//        String[] vals  = "null,android,ios".split(",");
//        FT100101(100101,"ft100101",20,100,1,"pos id combine",100),// 性别（百度/爱奇艺融合）
//        FT100102(100102,"ft100102",20,10000,1,"pos id combine",10000),// 年龄_百度
//        String[] vals  = "119010101,119010102,null".split(",");
//        String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        String[] vals = "77506_0,39882_0,19_0,39_0,77506_null,39882_null,19_null,39_null".split(",");
//        String[] vals = "1,2,3,4,5,6,7,8,9,10,11,12,13".split(",");
//
//        System.out.println((1 & 2));
//
//
        String[] vals  = "65521_0,79445_0".split(",");
        for (int i = 0; i < vals.length; i++) {
            System.out.println(vals[i] + " " + Arrays.toString(getSubFIdHash(vals[i], 1048576)));
//            System.out.println(vals[i] + " " +  getSubFId("f840110", vals[i], 1000));
        }



//        Integer size = 200;
//        int result = 0;
//        String value = "2021";
//        int len = value.length();
//        for (int i = 0; i < len; i++) {
//
//            result = 3 * result + value.charAt(i);
//
//        }
//
//        System.out.println(((size - 1) & result));

//        119010101
//        119010102
//        String feature = "ft100104";
//        Integer nums = 1;
//        Integer size = 200;
////        10001000
////        10001001
////        10001002
////        10001003
////        10001004
////        10001005,


//        {16:3771,0:1004,17:744,1:1881,2:2203,18:43,19:354}

//        int[] ret = new int[vals.length];
//        System.out.println(Arrays.toString(vals));
//        for (int i = 0; i < vals.length; i++) {
//
//            String str = vals[i];
//            str = std(str);
//            ret[i] = 0 * size + HashUtil.hash(str, size,3);
//        }
//
////        Arrays.sort(ret);
////        String str = null;
////        Integer nums = 1;
////        Integer size = 3;
////        str = std(str);
////        int[] ret = new int[nums];
////        for (int i = 0; i < nums; i++) {
////            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);
////
//////            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);
////
//////            addHashConflictTest(feature, str, ret[i] + 0L);
////        }
////        Arrays.sort(ret);
////
////
////
////
////
//        System.out.println(Arrays.toString(ret));


    }

    public static String std(String oStr) {
        if (oStr == null) {
            return null;
        }
        String lowStr = oStr.toLowerCase();
        if (lowStr.length() == 0 ||
                lowStr.equals("\\n") ||
                lowStr.equals("\n") ||
                lowStr.equals("null") ||
                lowStr.equals("none")
        ) {
            return null;
        }
        return lowStr;
    }

}

==> alg-example-1.56.54-20220222.020025-7-sources.jar: cn/com/duiba/nezha/alg/example/util/demo.java <==

package cn.com.duiba.nezha.alg.example.util;

import cn.com.duiba.nezha.alg.common.enums.DateStyle;
import cn.com.duiba.nezha.alg.common.util.DataUtil;
import cn.com.duiba.nezha.alg.common.util.LocalDateUtil;
import cn.com.duiba.nezha.alg.common.util.MathUtil;
import cn.com.duiba.nezha.alg.example.util.JedisUtil;
import cn.com.duiba.nezha.alg.example.util.conf.JedisPoolConf;
import cn.com.duiba.nezha.alg.common.util.StringZIP;
import cn.com.duiba.nezha.alg.feature.coder.FeatureCoderBase;
import cn.com.duiba.nezha.alg.feature.util.DataConverter;
import cn.com.duiba.nezha.alg.feature.util.HashUtil;
import com.alibaba.fastjson.JSON;
import cn.com.duiba.nezha.alg.example.util.conf.JedisConfig;
import redis.clients.jedis.Jedis;

import java.util.*;

import static cn.com.duiba.nezha.alg.feature.coder.FeatureCoderBase.getVectorSize;
import static java.util.stream.Collectors.joining;

public class demo {

    private static int F_MAX_SIZE = 64;

    public static int[] seed = {3, 7, 11, 17, 19, 31, 41};
    public static int getSubFId(String feature, String str, int size) {
        int ret = 0;
        str = std(str);

        try {
            if (str != null) {
                Integer tmp = Integer.valueOf(str);

                if (tmp != null && tmp >= 0) {
//                System.out.println(((size - 1) + tmp.intValue()) & (size - 1));
                    ret = ((size - 1) + tmp) % (size - 1);
                    ret += 1;
                }
            }
        } catch (Exception e) {
            System.out.println("errot");
        }

//        addHashConflictTest(feature, str, ret + 0L);
        return ret;
    }



//    public static int[] Arrandlist() {
//        List<String> list = new ArrayList<>();
//        list.add("apple"); // size=1
//        list.add("pear"); // size=2
//        list.add("apple"); // 允许重复添加元素，size=3
//
//
////        List<String> list = List.of("apple", "pear", "banana");
//
//    }


    public static int[] getSubFIdHash(String str, int size) {
        int nums = 1;
        str = std(str);
        int[] ret = new int[nums];
        for (int i = 0; i < nums; i++) {
            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);

//            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);

//            addHashConflictTest(feature, str, ret[i] + 0L);
        }
        Arrays.sort(ret);


        return ret;
    }


    public static int hash(String value,int size,int seed) {
        int result = 0;
        if (value == null) {
            value = "";
        }
        int len = value.length();
        for (int i = 0; i < len; i++) {
            result = seed * result + value.charAt(i);
        }

        return (size - 1) & result;
    }

    public static int[] getHashSubFId(String feature, String str, int size, int nums) throws Exception {

        if (nums > seed.length) {
            return null;
        }
        str = std(str);
        int[] ret = new int[nums];
        for (int i = 0; i < nums; i++) {
            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);

//            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);

//            addHashConflictTest(feature, str, ret[i] + 0L);
        }
        Arrays.sort(ret);


        return ret;
    }

    public static Set<Integer> getIds(int fSize, int pFNums, int[] sFIds, List<Integer> pLenList, List<Integer> indices, List<Double> values) throws Exception {
        Set<Integer> ret = new HashSet<>();
        int pLen = getVectorSize(pLenList);
        if (sFIds != null && sFIds.length > 0) {
            double weight = MathUtil.reciprocalOfN(sFIds.length);
            for (int i = 0; i < sFIds.length; i++) {
                Integer newId = pLen + sFIds[i];
                indices.add(newId);
                ret.add(newId);
                values.add(weight);
            }
        }
        pLenList.add(pLen + pFNums * fSize);
        return ret;
    }


    public static Long bucket2(Double value, Double[] bucketList) {
        long ret = 0;
        if (value != null && bucketList != null && bucketList.length > 0) {
            ret = bucketList.length + 1;
            for (int i = 0; i < bucketList.length; i++) {
                double bound = bucketList[i];

                if (value <= bound) {
                    ret = i + 1;
                    break;
                }
            }


        }
        return ret;
    }
    public static Double[] userCtrBucket = {0.05, 0.1, 0.15, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0};

    public static void main(String[] args) throws Exception {






//
//        JedisConfig jedisConf = new JedisConfig();
//        jedisConf.setIp("test.config.dui88.com");
//        jedisConf.setPort(6379);
//        jedisConf.setPassWord("duiba123");
//
//        JedisUtil jedis = new JedisUtil(jedisConf);
//        jedis.set("chengpeng001", "good");
//        System.out.println(jedis.get("chengpeng001"));







//        List<Integer> indices2 = Arrays.asList(1, 2, 3);
//        String b = indices2.toString();
//        System.out.println(b);
//        System.exit(1);

//        Set<Integer> s = new HashSet<>();
//        s.add(1);
//        s.add(3);
//        System.out.println(s.stream().map(String::valueOf).collect(joining("_")));
//        System.exit(1);

//        System.out.println(HashUtil.hash("10001002", 10000, seed[0]));
//        System.exit(0);
//
//
////        String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        List<Integer> indices = new ArrayList<>(F_MAX_SIZE);//field起始+field内编码
//        List<Double> values = new ArrayList<>(F_MAX_SIZE);
//        List<String> fields = new ArrayList<>(F_MAX_SIZE);
//        List<Integer> pLenList = new ArrayList<>(F_MAX_SIZE);//field起始
//        Set<Integer> ret = null;
//        ret = FeatureCoderBase.getHashSubId("f1", "119010102", 100, 1, pLenList, indices, values);
//        System.out.println(ret);
//        System.exit(1);

////        System.out.println(LocalDateUtil.getDayOfWeek("2021-12-21 17:39:18", DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
//        String[] vals = "null,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86".split(",");
//////        String[] vals = "null,0,1,2,3,4".split(",");
//////        String[] vals = "null,1162003,6208378,1603690,799304,442719,291344,203217,145671,108196,81844,63950,51597,157175,77654,75228,56720,44393,15661".split(",");
//////////                String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        String[] vals  = "-1,9427,9152,9277,8755,9326,9427,9277,9326,9277,9277,9431,9431,9277,9427,8743,9277,8743,8755,9427,9277,9311,8755,9372,9277,9427,9277,9326,9277,9277,9311".split(",");
//        for (int i = 0; i < vals.length; i++) {
//            System.out.println(vals[i] + " " +  getSubFId("f840110", vals[i], 100));
//        }
//        System.exit(0);

//        String[] vals  = "null,android,ios".split(",");
//        FT100101(100101,"ft100101",20,100,1,"pos id combine",100),// 性别（百度/爱奇艺融合）
//        FT100102(100102,"ft100102",20,10000,1,"pos id combine",10000),// 年龄_百度
//        String[] vals  = "119010101,119010102,null".split(",");
//        String[] vals  = "null,-1,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021".split(",");
//        String[] vals = "77506_0,39882_0,19_0,39_0,77506_null,39882_null,19_null,39_null".split(",");
//        String[] vals = "1,2,3,4,5,6,7,8,9,10,11,12,13".split(",");
//
//        System.out.println((1 & 2));
//
//
//        String[] vals  = "65521_0,79445_0".split(",");
//        for (int i = 0; i < vals.length; i++) {
//            System.out.println(vals[i] + " " + Arrays.toString(getSubFIdHash(vals[i], 1048576)));
////            System.out.println(vals[i] + " " +  getSubFId("f840110", vals[i], 1000));
//        }
        Double[] ret = {null, 0.01,0.02,0.05,0.6,0.1,0.9};

        for (Double v : ret) {
            System.out.println(v);
            System.out.println(DataConverter.bucket(v, userCtrBucket));
            System.out.println(bucket2(v, userCtrBucket));
            System.out.println("====");

        }






//        Integer size = 200;
//        int result = 0;
//        String value = "2021";
//        int len = value.length();
//        for (int i = 0; i < len; i++) {
//
//            result = 3 * result + value.charAt(i);
//
//        }
//
//        System.out.println(((size - 1) & result));

//        119010101
//        119010102
//        String feature = "ft100104";
//        Integer nums = 1;
//        Integer size = 200;
////        10001000
////        10001001
////        10001002
////        10001003
////        10001004
////        10001005,


//        {16:3771,0:1004,17:744,1:1881,2:2203,18:43,19:354}

//        int[] ret = new int[vals.length];
//        System.out.println(Arrays.toString(vals));
//        for (int i = 0; i < vals.length; i++) {
//
//            String str = vals[i];
//            str = std(str);
//            ret[i] = 0 * size + HashUtil.hash(str, size,3);
//        }
//
////        Arrays.sort(ret);
////        String str = null;
////        Integer nums = 1;
////        Integer size = 3;
////        str = std(str);
////        int[] ret = new int[nums];
////        for (int i = 0; i < nums; i++) {
////            ret[i] = i * size + HashUtil.hash(str, size, seed[i]);
////
//////            System.out.println("featureId="+feature+",str="+str+",ret="+ret[i]);
////
//////            addHashConflictTest(feature, str, ret[i] + 0L);
////        }
////        Arrays.sort(ret);
////
////
////
////
////
//        System.out.println(Arrays.toString(ret));


    }

    public static String std(String oStr) {
        if (oStr == null) {
            return null;
        }
        String lowStr = oStr.toLowerCase();
        if (lowStr.length() == 0 ||
                lowStr.equals("\\n") ||
                lowStr.equals("\n") ||
                lowStr.equals("null") ||
                lowStr.equals("none")
        ) {
            return null;
        }
        return lowStr;
    }

}
